package ru.dbotthepony.mc.otm.menu.decorative

import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import ru.dbotthepony.mc.otm.block.entity.decorative.CargoCrateBlockEntity
import ru.dbotthepony.mc.otm.container.slotted.SlottedContainer
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.QuickMoveInput
import ru.dbotthepony.mc.otm.menu.SortInput
import ru.dbotthepony.mc.otm.menu.UserFilteredMenuSlot
import ru.dbotthepony.mc.otm.menu.makeSlots
import ru.dbotthepony.mc.otm.registry.game.MMenus

class CargoCrateMenu(
	containerId: Int,
	inventory: Inventory,
	tile: CargoCrateBlockEntity? = null
) : MatteryMenu(MMenus.CARGO_CRATE, containerId, inventory, tile) {
	val actualContainer = tile?.container ?: SlottedContainer.filtered(CargoCrateBlockEntity.CAPACITY)
	val storageSlots = makeSlots(actualContainer, ::UserFilteredMenuSlot)
	private val trackedPlayerOpen = !inventory.player.isSpectator

	val sort = SortInput(this, actualContainer, playerSortSettings)
	val quickMoveToStorage = QuickMoveInput.create(this, playerCombinedInventorySlots, storageSlots)
	val quickMoveFromStorage = QuickMoveInput.create(this, storageSlots, playerInventorySlots)

	init {
		if (trackedPlayerOpen) {
			tile?.onPlayerOpen()
		}

		addStorageSlot(storageSlots)
		addInventorySlots()
	}

	override fun removed(p_38940_: Player) {
		super.removed(p_38940_)

		if (trackedPlayerOpen) {
			(tile as? CargoCrateBlockEntity)?.onPlayerClose()
		}
	}
}
